#
#

#-------------------------------------------------------------------------------

BEGIN {
   mask_no = -1;
   field_no = -1;
}

#-------------------------------------------------------------------------------

$1 == "FILE" {
   header_file = $2;
   form_file = $3;
   form_script = $4;
   form_title = substr( $0, index( $0, $5 ) );
}

$1 == "FORM_BG" {
   form_bg = $2;
}

$1 == "FORM_HDR" {
   form_hdr = substr( $0, index( $0, $2 ) );
}

$1 == "FORM_INTRO" {
   form_intro = substr( $0, index( $0, $2 ) );
}

$1 == "FORM_FTR" {
   form_ftr = substr( $0, index( $0, $2 ) );
}

#-------------------------------------------------------------------------------

$1 == "FIELD" {
   field_no++;
   field_type[field_no] = $2;
   field_name[field_no] = $3;
   field_label[field_no] = substr( $0, index( $0, $4 ) );
}

$1 == "FORM_FIRST" {
   first_y[field_no] = $2 + 0;
   first_m[field_no] = $3 + 0;
   first_d[field_no] = $4 + 0;
}

$1 == "FORM_LAST" {
   last_y[field_no] = $2 + 0;
   last_m[field_no] = $3 + 0;
   last_d[field_no] = $4 + 0;
}

$1 == "FORM_SCROLL" {
   mask_type[field_no] = "scroll";
   if( $2 == "SINGLE" )
      mask_args[field_no] = "";
   else
      mask_args[field_no] = " MULTIPLE";
}

$1 == "FORM_CHECK" {
   if( $2 == "SINGLE" )
      mask_type[field_no] = "radio";
   else
      mask_type[field_no] = "checkbox";
}

$1 == "FORM_EXCLUDE" {
   exclude[field_no] = 1;
}

$1 == "BITS" {
   field_mask[field_no] = $2;
}

$1 == "MASK" {
   bit = -1;
   mask_no++;
   mask_name[mask_no] = $2;
   mask_i[$2] = mask_no;
   mask_summary[mask_no] = -1;
   submask = -1;

   if( $3 == "SUMMARY" ) {
      for( i = 0; i < mask_no; i++ )
      {
         if( mask_name[i] == $4 ) {
            mask_summary[mask_no] = i;
            break;
         }
      }
   }
}

$1 == "MBIT" {
   bit++;
   mask_label[mask_no,bit] = substr( $0, index( $0, $2 ) );

   if( mask_summary[mask_no] == -1 ) {
      if( bit == 0 ) {
         mask_value[mask_no,bit] = 1;
      }
      else {
         bm1 = bit - 1;
         mask_value[mask_no,bit] = mask_value[mask_no,bm1] * 2;
      }
   }
   else {
      if( submask > -1 ) {
         submask_len[mask_no,bit] = submask;
      }
      submask = -1;
      mask_value[mask_no,bit] = 0;
   }
}

$1 == "KSAM" {
   mask_len[mask_no] = bit + 1;
   if( submask > -1 ) {
      submask_len[mask_no,bit] = submask;
   }
}

$1 == "VALUE" {
   i = mask_summary[mask_no];
   label = substr( $0, index( $0, $2 ) );
   submask++;

   for( j = 0; j <= mask_len[i]; j++ )
   {
      if( mask_label[i,j] == label )
      {
         mask_value[mask_no,bit] += mask_value[i,j];
         submask_i[mask_no,bit,submask] = j;
      }
   }
}

#-------------------------------------------------------------------------------

END {

   #  Mask strings.
   #
   for ( i = 0; i <= mask_no; i++ ) {
      printf( "static char* %s[] = {\n", mask_name[i] ) > header_file;
      for ( j = 0; j < mask_len[i]; j++ ) {
         printf( " \"%s\",\n", mask_label[i,j] ) > header_file;
      }
      print " 0\n};" > header_file;
   }

   #  Mask lookup.
   #
   print "static DBMask masks[] = {" > header_file;
   for ( i = 0; i <= field_no; i++ ) {
      if( field_type[i] == "MASK" || field_type[i] == "LOOKUP" ) {
         m = mask_i[field_mask[i]];
         printf( " { \"%s\", %s },\n", \
         field_name[i], mask_name[m] ) > header_file;
      }
   }
   print " { \"\", 0 }" > header_file;
   print "};" > header_file;

   #  DB Schema.
   #
   print "static DBField fields[] = {" > header_file;
   for ( i = 0; i <= field_no; i++ ) {
      printf( " { %s, \"%s\" },\n", \
      field_type[i], field_name[i] ) > header_file;
   }
   print " { LAST, 0 }" > header_file;
   print "};" > header_file;

   #  Variables to hold "current" record.
   #
   for ( i = 0; i <= field_no; i++ ) {
      n = 16;
      if (field_type[i] == "STRING" ) n = 255;
      printf( "static char f_%s[%d];\n", field_name[i], n ) > header_file;
   }

   #  Function to load values into field variables.
   #
   print "void LoadVariables( void ) {" > header_file;
   for ( i = 0; i <= field_no; i++ ) {
      printf( "   dbValue( %d, f_%s );\n", i, field_name[i] ) > header_file;
   }
   print "}" > header_file;

   #  Form header.
   #
   print "<html>" > form_file;
   printf( "<header><title>%s</title></header>\n", form_title ) > form_file;
   if( form_bg != "" )
      print "<body background=\"" form_bg "\">" > form_file;
   else
      print "<body>" > form_file;
   if( form_hdr != "" )
      print form_hdr > form_file;
   printf( "<h2>%s</h2>\n", form_title ) > form_file;
   if( form_intro != "" )
      print form_intro > form_file;
   printf( "<form method=\"post\" action=\"%s\">\n", form_script ) > form_file;
   print "<input type=\"submit\" value=\"Search\">" > form_file;
   print "<input type=\"reset\" value=\"Clear Selections\">" > form_file;
   print "<br><hr>" > form_file;

   #  Form body.
   #
   for ( i = 0; i <= field_no; i++ ) {
      if( exclude[i] ) continue;
      printf( "<p><font size=\"+1\">%s:</font>\n", field_label[i] ) \
      > form_file;

      if( field_type[i] == "MASK" ) {
         if( mask_type[i] == "scroll" ) {
            m = mask_i[field_mask[i]];
            size = mask_len[m];
            if( size > 8 ) size = 8;

            ms = mask_summary[m];
            if ( ms != -1) {
               for( j = 0; j <= field_no ; j++ ) {
                  if ( mask_i[field_mask[j]] == ms ) { break; }
               }
               ms = j;
            }
            else {
               ms = i;
            }

            printf( "<select name=\"%s\"%s SIZE=\"%d\"><br>\n",
            field_name[ms], mask_args[i], size ) > form_file;
            for ( j = 0; j < mask_len[m]; j++ ) {
               printf( " <option value=\"0x%08x\">%s\n", \
                  mask_value[m,j], mask_label[m,j] ) > form_file;
            }
            print "</select>" > form_file;
         }
         else if( mask_type[i] == "radio" || mask_type[i] == "checkbox" ) {
            m = mask_i[field_mask[i]];

            ms = mask_summary[m];
            if ( ms == -1) { ms = m; }

            for ( j = 0; j < mask_len[m]; j++ ) {
               printf( " <input type=\"%s\" name=\"%s\"", \
               mask_type[i], field_name[ms] ) > form_file;
               printf( " value=\"0x%08x\"> %s\n", \
                  mask_value[m,j], mask_label[m,j] ) > form_file;
            }
         }
      }
      else if( field_type[i] == "LOOKUP" ) {
         if( mask_type[i] == "scroll" ) {
            m = mask_i[field_mask[i]];
            size = mask_len[m];
            if( size > 8 ) size = 8;

            ms = mask_summary[m];
            if ( ms != -1) {
               for( j = 0; j <= field_no ; j++ ) {
                  if ( mask_i[field_mask[j]] == ms ) { break; }
               }
               ms = j;
            }
            else {
               ms = i;
            }

            printf( "<select name=\"%s\"%s SIZE=\"%d\"><br>\n",
            field_name[ms], mask_args[i], size ) > form_file;
            for ( j = 0; j < mask_len[m]; j++ ) {
               printf( " <option value=\"%d\">%s\n", \
                  j, mask_label[m,j] ) > form_file;
            }
            print "</select>" > form_file;
         }
         else if( mask_type[i] == "radio" || mask_type[i] == "checkbox" ) {
            m = mask_i[field_mask[i]];

            ms = mask_summary[m];
            if ( ms == -1) {
               for ( j = 0; j < mask_len[m]; j++ ) {
                  printf( " <input type=\"%s\" name=\"%s\"", \
                  mask_type[i], field_name[i] ) > form_file;
                  printf( " value=\"%d\"> %s\n", \
                     j, mask_label[m,j] ) > form_file;
               }
            }
            else
            {
               for ( j = 0; j < mask_len[m]; j++ ) {
                  printf( " <input type=\"%s\" name=\"%s\"", \
                  mask_type[i], field_name[ms] ) > form_file;
                  printf( " value=\"0x%08x\"> %s\n", \
                     mask_value[m,j], mask_label[m,j] ) > form_file;
               }
            }
         }
      }
      else if( field_type[i] == "STRING" ) {
         printf( "<input name=\"%s\"><br>\n", field_name[i] ) > form_file;
      }
      else if( field_type[i] == "DATE" ) {
         #DateComponent( "Year",  "y", field_name[i] );
         #DateComponent( "Month", "m", field_name[i] );
         #DateComponent( "Day",   "d", field_name[i] );
         BuildDateCompar( field_name[i] );
         printf( "<input name=\"%s\" SIZE=\"8\" MAXLENGTH=\"8\"> MM/DD/YY\n",\
            field_name[i] ) > form_file;
         #BuildDateList( first_y[i], last_y[i], "Year",  field_name[i] );
         #BuildDateList( first_m[i], last_m[i], "Month", field_name[i] );
         #BuildDateList( first_d[i], last_d[i], "Day",   field_name[i] );
         #print "<br>" > form_file;
      }
   }

   #  Form footer.
   #
   print "</form>" > form_file;
   if( form_ftr != "" )
      print form_ftr > form_file;
   print "</body>" > form_file;
   print "</html>" > form_file;
}

#-------------------------------------------------------------------------------

function BuildDateCompar( name ) {
   printf( "<select name=\"%s_compar\">\n", name ) > form_file;
   print "<option value=\"1\">EQ"          > form_file;
   print "<option value=\"2\">NE"          > form_file;
   print "<option value=\"3\">LT"          > form_file;
   print "<option value=\"4\">LE"          > form_file;
   print "<option value=\"5\">GT"          > form_file;
   print "<option value=\"6\" selected>GE" > form_file;
   print "</select>" > form_file;
}

function DateComponent( label, suffix, name ) {
   printf( "%s: <input name=\"%s_%s\" SIZE=\"4\" MAXLENGTH=\"4\">\n", \
   label, name, suffix ) > form_file;
}

function BuildDateList( first, last, label, name, size, v, i ) {
   size = last - first + 1;
   if( size > 8 ) size = 8;
   printf( "%s: ", label ) > form_file;
   printf( "<select name=\"%s_y\" MULTIPLE SIZE=\"%d\">\n",\
   name, size ) > form_file;
   v = 1;
   for( i = first; i <= last; i++ ) {
      printf( " <option value=\"0x%08x\">%s\n", v, i ) > form_file;
      v *= 2;
   }
   print "</select>" > form_file
}

#-------------------------------------------------------------------------------
